import { LatLng, Point } from 'leaflet';
import MyMap from '../Map';

export default class MapUtil {
    /** 
     * 给定地理坐标，转换为相对于origin pixel的相应像素坐标。（在地图上进行位置叠加时比较有用）
     * @param {{lat: number, lng: number}} latlng - 经纬度
     * @returns {{x:number, y:number}} - 像素点坐标
     */
    static latLngToLayerPoint(latlng){
        if(!MyMap.map){ console.error("请先初始化地图!"); return; }
        let ll = new LatLng(latlng.lat, latlng.lng);
        let p = MyMap.map.latLngToLayerPoint(ll);
        return {x: p.x, y: p.y};
    }

    /** 
     * 给定相对于origin pixel的相应像素坐标，转换为相应的地理坐标（对于当前缩放级别）。
     * @param {{x:number, y:number}} point - 像素点坐标
     * @returns {{lat: number, lng: number}} - 经纬度
     */
    static layerPointToLatLng(point){
        if(!MyMap.map){ console.error("请先初始化地图!"); return; }
        let p = new Point(point.x, point.y);
        let ll = MyMap.map.layerPointToLatLng(p);
        return {lat: ll.lat, lng: ll.lng};
    }

    /** 
     * 给给定地理坐标，返回相对于地图container容器的相应像素坐标。
     * @param {{lat: number, lng: number}} latlng - 经纬度
     * @returns {{x:number, y:number}} - 像素点坐标
     */
    static latLngToContainerPoint(latlng){
        if(!MyMap.map){ console.error("请先初始化地图!"); return; }
        let ll = new LatLng(latlng.lat, latlng.lng);
        let p = MyMap.map.latLngToContainerPoint(ll);
        return {x: p.x, y: p.y};
    }

    /** 
     * 给定相对于地图container容器的像素坐标，返回对应的地理坐标（对于当前缩放级别）。
     * @param {{x:number, y:number}} point - 像素点坐标
     * @returns {{lat: number, lng: number}} - 经纬度
     */
    static containerPointToLatLng(point){
        if(!MyMap.map){ console.error("请先初始化地图!"); return; }
        let p = new Point(point.x, point.y);
        let ll = MyMap.map.containerPointToLatLng(p);
        return {lat: ll.lat, lng: ll.lng};
    }

    /** 
     * 将地理坐标投影为接受单位的坐标 此 CRS（例如 EPSG:3857 的仪表，用于将其传递给 WMS 服务）。
     * @param {{lat: number, lng: number}} latlng - 经纬度
     * @param {number} zoom - 缩放级别
     * @returns {{x:number, y:number}} - 像素点坐标
     */
    static project(latlng, zoom){
        if(!MyMap.map){ console.error("请先初始化地图!"); return; }
        let p = MyMap.map.project(latlng, zoom);
        return {x: p.x, y: p.y};
    }

    /** 
     * 给定一个投影坐标返回相应的 LatLng。。
     * @param {{x:number, y:number}} point - 像素点坐标
     * @param {number} zoom - 缩放级别
     * @returns {{lat: number, lng: number}} - 经纬度
     */
    static unproject(point, zoom){
        if(!MyMap.map){ console.error("请先初始化地图!"); return; }
        let ll = MyMap.map.unproject(point, zoom);
        return {lat: ll.lat, lng: ll.lng};
    }

    /** 
     * 返回将投影坐标转换为特定缩放的像素坐标时使用的比例。
     * @param {number} toZoom - 缩放级别
     * @param {number} fromZoom - 缩放级别
     * @returns {number} - 比例因子
     */
    static scale(toZoom, fromZoom){
        if(!MyMap.map){ console.error("请先初始化地图!"); return; }
        return MyMap.map.getZoomScale(toZoom, fromZoom);
    }

    /** 
     * 返回与 scale的比例因子对应的缩放级别。
     * @param {number} scale - 比例因子
     * @param {number} fromZoom - 缩放级别
     * @returns {number} - 缩放级别
     */
    static zoom(scale, fromZoom){
        if(!MyMap.map){ console.error("请先初始化地图!"); return; }
        return MyMap.map.getScaleZoom(scale, fromZoom);
    }

    /** 
     * 返回两个地理坐标之间的距离
     * @param {{lat: number, lng: number}} latlng1 - 经纬度1
     * @param {{lat: number, lng: number}} latlng2 - 经纬度2
     * @returns {number} - 距离（单位：米）
     */
    static distance(latlng1, latlng2){       
        if(!MyMap.map){ console.error("请先初始化地图!"); return; } 
		let ll1 = new LatLng(latlng1.lat, latlng1.lng);
		let ll2 = new LatLng(latlng2.lat, latlng2.lng);
		// let distance = ll1.distanceTo(ll2) / 1000;
        return MyMap.map.distance(ll1, ll2);
    }

    /** 
     * 地理坐标移动x,y距离后的地理坐标
     * @param {{lat: number, lng: number}} latlng - 经纬度
     * @param {number} distX - x移动距离（单位：米）
     * @param {number} distY - y移动距离（单位：米）
     * @returns {{lat: number, lng: number}} - 经纬度
     */
    static moveLatLng(latlng, distX, distY) {
        const R = 6378137; // Radius of the Earth in meters
    
        // Convert distances from meters to radians
        const dLat = distY / R;
        const dLng = distX / (R * Math.cos(latlng.lat * Math.PI / 180));
    
        // Calculate new latitude and longitude
        const newLat = latlng.lat + (dLat * 180 / Math.PI);
        const newLng = latlng.lng + (dLng * 180 / Math.PI);
    
        return {lat: newLat, lng: newLng};
    }
}